#include <stdio.h>
#include <errno.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>

#include "irmp3io.h"

int irmp3_io_connect(char *remote_host)
{
	int					iSocket;
	struct sockaddr_in HostSockAddr;
	struct hostent		*Host;
	int					iRetstat;
	struct	servent	*pseService;

	iSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (iSocket == -1)
	{
		perror("irmp3-monitor");
		return -1;
	}
	setsockopt(iSocket, SOL_SOCKET, SO_KEEPALIVE, 0, 0);

	if (!remote_host)
		remote_host = "localhost";
	Host = gethostbyname(remote_host);
	if (!Host)
	{
		perror("irmp3-monitor");
		return -2;
	}

	HostSockAddr.sin_family = Host->h_addrtype;
	HostSockAddr.sin_addr = *((struct in_addr *)Host->h_addr);

	pseService = getservbyname(	"irmp3", "tcp");
	if (pseService)
	{
		HostSockAddr.sin_port = pseService->s_port;
	}
	else
	{
		HostSockAddr.sin_port = htons((short)atoi("9232"));
		if (!HostSockAddr.sin_port)
		{
			printf("Unable to find service in database");
			return 0;
		}
	}
	if ((iRetstat = connect (iSocket, (struct sockaddr *)&HostSockAddr, sizeof(struct sockaddr_in))))
	{
		perror("irmp3-monitor");
		return -3;
	}

	write(iSocket, "mixer volume +0\r\n", sizeof("mixer volume +0\r\n"));
	return iSocket;
}

#define	MSGTYPE_VERSION	100
#define	MSGTYPE_CLIENTID	110
#define MSGTYPE_GENERIC	200
#define	MSGTYPE_INPUT	210
#define MSGTYPE_MIXER	220
#define	MSGTYPE_PLAYER	230

int irmp3_io_command(char *message, void *data)
{
	char	*pArgs[20];
	char	buf[1024];
	int		iArgCount;
	int		iMsgType;
	strcpy(buf, message);
	ParseLine(buf, pArgs, &iArgCount, 20);
	if (iArgCount > 1)
	{
		iMsgType = atoi(pArgs[0]);
		switch (iMsgType)
		{
			case	MSGTYPE_VERSION:
				break;
			case	MSGTYPE_CLIENTID:
				break;
			case	MSGTYPE_PLAYER:
				if (iArgCount >= 2)
				{
					if (strcmp(pArgs[1], "time") == 0)
					{
						int	iAmountDone = atoi(pArgs[2]);
						int	iAmountToDo = atoi(pArgs[3]);
						irmp3_io_callback_time(data, iAmountDone, iAmountToDo);
						// printf("%f\n", (gfloat)iAmountDone / (gfloat)(iAmountDone + iAmountToDo));
						//gtk_progress_set_percentage(pPB, (gfloat)iAmountDone / (gfloat)(iAmountDone + iAmountToDo));
					}
					else if (strcmp(pArgs[1], "info") == 0)
					{
						if (strcmp(pArgs[2], "artist") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_artist(data, message + (pArgs[3] - buf));
							}
						}
						if (strcmp(pArgs[2], "title") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_title(data, message + (pArgs[3] - buf));
							}
						}
						if (strcmp(pArgs[2], "genre") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_genre(data, message + (pArgs[3] - buf));
							}
						}
						if (strcmp(pArgs[2], "album") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_album(data, message + (pArgs[3] - buf));
							}
						}
						if (strcmp(pArgs[2], "year") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_year(data, message + (pArgs[3] - buf));
							}
						}
						if (strcmp(pArgs[2], "comment") == 0)
						{
							if (iArgCount > 3)
							{
								irmp3_io_callback_comment(data, message + (pArgs[3] - buf));
							}
						}
					}
					else if (strcmp(pArgs[1], "endofsong") == 0)
					{
						irmp3_io_callback_endofsong(data);
					}
					else if (strcmp(pArgs[1], "play") == 0)
					{
						irmp3_io_callback_beginofsong(data);
					}
					else if (strcmp(pArgs[1], "pause") == 0)
					{
						irmp3_io_callback_pause(data);
					}
					else if (strcmp(pArgs[1], "stop") == 0)
					{
						irmp3_io_callback_stop(data);
					}
					else if (strcmp(pArgs[1], "version") == 0)
					{
//						irmp3_io_callback_stop(data);
					}
					else
					{
						printf("got unknown message: %s\n", message);
					}
				}
				break;
			case	MSGTYPE_GENERIC:
				break;
			case	MSGTYPE_INPUT:
				if (iArgCount >= 2)
				{
					if (strcmp(pArgs[1], "play") == 0)
					{
						irmp3_io_callback_file(data, message + (pArgs[2] - buf));
					}
					else if (strcmp(pArgs[1], "mixer") == 0)
					{
					}
					else if (strcmp(pArgs[1], "playlist") == 0)
					{
					}
					else if (strcmp(pArgs[1], "query") == 0)
					{
					}
					else if (strcmp(pArgs[1], "stop") == 0)
					{
					}
					else if (strcmp(pArgs[1], "loadplaylist") == 0)
					{
						irmp3_io_callback_playlist(data, message + (pArgs[2] - buf));
					}
					else
					{
						printf("got unknown message: %s\n", message);
					}
				}
				break;
			case	MSGTYPE_MIXER:
				if (strcmp(pArgs[1], "volume") == 0)
				{
					irmp3_io_callback_volume(data, atoi(pArgs[2]));
				}
				else if (strcmp(pArgs[1], "info") == 0)
				{
					if (strcmp(pArgs[2], "volume") == 0)
					{
						irmp3_io_callback_volume(data, atoi(pArgs[3]));
					}
				}
				else if (strcmp(pArgs[1], "channel") == 0)
				{
					irmp3_io_callback_channel(data, pArgs[2], atoi(pArgs[3]));
				}
				else if (strcmp(pArgs[1], "mute") == 0)
				{
					irmp3_io_callback_mute(data, atoi(pArgs[2]));
				}
				break;
			default:
				break;
		}
	}
}

int    irmp3_io_data (int iSocket, void *data)
{
	static char	buf[1024];
	static int	iPos = 0;
	int		ii;
	int	iLen;
	iLen = read(iSocket, buf + iPos, sizeof(buf) - iPos);
	for (ii = iPos; ii < iPos + iLen; ii++)
	{
		if ((buf[ii] == '\r') || (buf[ii] == '\n'))
			break;
	}
	if (ii < (iPos + iLen))	// end of message found
	{
		iPos = iPos + iLen;
		while (1)
		{
			// printf("Loop ii = %d, iPos = %d, iLen = %d\n", ii, iPos, iLen);
			buf[ii] = 0;
			ii++;
			irmp3_io_command(buf, data);
			while (isspace(buf[ii]) && (ii < iPos))
				ii++;
			if (ii >= iPos)
			{
				iPos = 0;
				break;
			}
			memcpy(buf, buf + ii, iPos - ii);
			iPos = iPos - ii;
			for (ii = 0; ii < iPos; ii++)
			{
				if ((buf[ii] == '\r') || (buf[ii] == '\n'))
					break;
			}
			if (ii >= iPos)
			{
				iPos = 0;
				break;
			}
		}
	}
	else
	{
		iPos = iPos + iLen;
	}
}
