#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "irmp3io.h"

#define LEFTALIGN 1
#define CENTERALIGN 2
#define RIGHTALIGN 4

TTF_Font *font;
void RenderStatusText(SDL_Surface* surface, TTF_Font* font, const char texttorender[255], int xpos, int ypos, int width, int align, int anti, int blank);
void do_periodic(SDL_Surface *);

int main (int argc, char *argv[])
{
	int iSocket;
	SDL_Surface *screen;
	SDL_Surface *statussurface;
	SDL_Event	event;
	fd_set	fds;
	struct timeval	tv;
	TTF_Font *smallfont;
	char	bQuit = 0;

	/* Initialize the SDL library */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}

	/* Initialize the TTF library */
	if ( TTF_Init() < 0 ) {
			fprintf(stderr, "Couldn't initialize TTF: %s\n",SDL_GetError());
			exit(2);
	}
	atexit(SDL_Quit);

	screen = SDL_SetVideoMode(640, 480, 16, SDL_ANYFORMAT);
	if ( screen == NULL ) {
		fprintf(stderr, "Couldn't set 640x480x16 video mode: %s\n",
		SDL_GetError());
		exit(1);
	}

	//Blank the screen.
	SDL_FillRect(screen, NULL, SDL_MapRGBA(screen->format, 0, 0, 0, 0));

	statussurface = SDL_CreateRGBSurface(SDL_HWSURFACE, 640, 480, 16, 0, 0, 0, 0);

	font = TTF_OpenFont("comic.ttf", 20);
	smallfont = TTF_OpenFont("comic.ttf", 12);

	RenderStatusText(statussurface, smallfont, "Artist:", 100, 110, 100, RIGHTALIGN, 1, 1);
	RenderStatusText(statussurface, smallfont, "Album:", 100, 135, 100, RIGHTALIGN, 1, 1);
	RenderStatusText(statussurface, smallfont, "Title:", 100, 160, 100, RIGHTALIGN, 1, 1);
	SDL_BlitSurface(statussurface, NULL, screen, NULL);
		SDL_Flip(screen);
	iSocket = irmp3_io_connect(argv[1]);
	while (bQuit == 0)
	{
		FD_ZERO(&fds);
		FD_SET(iSocket, &fds);

		tv.tv_sec = 0;
		tv.tv_usec = 15000;
		if (select(iSocket + 1, &fds, 0, 0, &tv) == 0)
		{
			do_periodic(statussurface);
		}
		if (FD_ISSET(iSocket, &fds))
		{
			irmp3_io_data(iSocket, statussurface);
		}
		SDL_PollEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
				bQuit = 1;
				break;
			default:
				break;
		}
		SDL_BlitSurface(statussurface, NULL, screen, NULL);
		SDL_Flip(screen);
	}
	
	TTF_CloseFont(font);
	TTF_CloseFont(smallfont);
	SDL_Quit();
}

void do_periodic(SDL_Surface *surface)
{
	static int iPos;
	static char	direction = 3;

	SDL_Rect	pb_dark;
	SDL_Rect	pb_light;

	pb_dark.x = 0;
	pb_dark.y = 0;
	pb_dark.w = 640;
	pb_dark.h = 5;

	SDL_FillRect(surface, &pb_dark, SDL_MapRGBA(surface->format, 0, 50, 50, 0));

	pb_light.x = iPos;;
	pb_light.y = 0;
	pb_light.w = 25;
	pb_light.h = 5;

	SDL_FillRect(surface, &pb_light, SDL_MapRGBA(surface->format, 0, 150, 150, 0));
	iPos += direction;
	if (iPos >= 615)
	{
		direction = -3;
	}
	else if (iPos <= 0)
	{
		direction = 3;
	}
}

void irmp3_io_callback_artist(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 100, 100, 540, LEFTALIGN, 1, 1);
}

void irmp3_io_callback_album(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 100, 125, 540, LEFTALIGN, 1, 1);
}

void irmp3_io_callback_title(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 100, 150, 540, LEFTALIGN, 1, 1);
}

void irmp3_io_callback_year(void *data, char *text)
{
}

void irmp3_io_callback_comment(void *data, char *text)
{
}

void irmp3_io_callback_endofsong(void *data)
{
}

void irmp3_io_callback_file(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 0, 225, 640, LEFTALIGN, 1, 1);
}

void irmp3_io_callback_playlist(void *data, char *text)
{
}

void irmp3_io_callback_volume(void *data, int iVolume)
{
}

void irmp3_io_callback_time(void *data, int amount_done, int amount_todo)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	char	caText[20];

	SDL_Rect	pb_dark;
	SDL_Rect	pb_light;

	pb_dark.x = 100;
	pb_dark.y = 177;
	pb_dark.w = amount_done * 440 / (amount_done + amount_todo);
	pb_dark.h = 25;

	SDL_FillRect(surface, &pb_dark, SDL_MapRGBA(surface->format, 0, 50, 50, 0));

	pb_light.x = pb_dark.w + 101;
	pb_light.y = 177;
	pb_light.w = 439 - pb_dark.w;
	pb_light.h = 25;

	SDL_FillRect(surface, &pb_light, SDL_MapRGBA(surface->format, 0, 150, 150, 0));

	sprintf(caText, "%d:%02d (%d:%02d)", amount_done / 60, amount_done % 60, amount_todo / 60, amount_todo % 60);
	RenderStatusText(surface, font, caText, 320, 175, 640, CENTERALIGN, 1, 0);
}

void irmp3_io_callback_genre(void *data, char *text)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	RenderStatusText(surface, font, text, 320, 200, 640, CENTERALIGN, 1, 1);
}

void irmp3_io_callback_beginofsong(void *data)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	SDL_Surface	*icon;
	SDL_Rect	dest;
	icon = SDL_LoadBMP("play.bmp");

	dest.x = 100;
	dest.y = 50;
	dest.w = icon->w;
	dest.h = icon->h;
	
	SDL_BlitSurface(icon, NULL, surface, &dest);
}

void irmp3_io_callback_pause(void *data)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	SDL_Surface	*icon;
	SDL_Rect	dest;
	icon = SDL_LoadBMP("pause.bmp");

	dest.x = 100;
	dest.y = 50;
	dest.w = 32;
	dest.h = 32;
	
	SDL_BlitSurface(icon, NULL, surface, &dest);
}

void irmp3_io_callback_stop(void *data)
{
	SDL_Surface *surface = (SDL_Surface *)data;
	SDL_Surface	*icon;
	SDL_Rect	dest;
	icon = SDL_LoadBMP("stop.bmp");

	dest.x = 100;
	dest.y = 50;
	dest.w = 32;
	dest.h = 32;
	
	SDL_BlitSurface(icon, NULL, surface, &dest);
}

void irmp3_io_callback_channel(void *data, char *text, int level)
{
}

void irmp3_io_callback_mute(void *data, int status)
{
}

void RenderStatusText(SDL_Surface* surface, TTF_Font* font, const char texttorender[255], int xpos, int ypos, int width, int align, int anti, int blank)
{
	SDL_Color* fg;
	SDL_Color* bg;
	SDL_Rect back;
	SDL_Surface* text;
	SDL_Color foregr = {255,255,255,255};
	SDL_Color backgr = {0,0,0,255};

	fg = &foregr;
	bg = &backgr;

	if (anti == 0)
	{
		text = TTF_RenderText_Shaded(font, texttorender, *fg, *bg);
	}
	else
	{
		text = TTF_RenderText_Blended(font, texttorender, *fg);
	}
	
	if (text != NULL)
	{
		if (align == CENTERALIGN)
		{
			back.x = xpos - (width/2);
		}
		else if (align == RIGHTALIGN)
		{
			back.x = xpos - width;
		}
		else
		{
			back.x = xpos;
		}
		back.y = ypos;
		back.h = text->h;
		back.w = width;
		if (blank)
		{
			SDL_FillRect(surface, &back, SDL_MapRGBA(surface->format, 0, 0, 0, 0));
		}

		back.w = text->w;
		if (align == CENTERALIGN)
		{
			back.x = xpos - (text->w / 2);
		}
		else if (align == RIGHTALIGN)
		{
			back.x = xpos - text->w;
		}
		else
		{
			back.x = xpos;
		}

		SDL_BlitSurface(text, NULL, surface, &back);

		SDL_FreeSurface(text);
	}
}
